#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/io.h>
#include "m68k.h"
#include "doorsos.h"
#include "objimage.h"
#include "tiformat.h"
#include "link.h"

int    nbsym,nbrel,nbok;
int    snb;
// struct external_reloc* rel;
struct external_filehdr* hdr;
struct external_scnhdr* scnhdr;  // section header
struct external_syment* symb;    // symbol
//struct external_reloc* rel;      // reloc entry
char *strtable;                  // String Table

char *project = 0;

BOOL quiet;

/* fonction read_object
 Reads a .o object file and loads it into memory
 For COFF:
  - Checks the magic number, the numbers of sections
    the size of the optional header.
 For AmigaDOS:
  - Just checks that the file is a valid AmigaDOS file
    */

objfile *read_object(char* name)
{
 FILE *of;
 objfile *object;

 of = fopen(name,"rb");
 object = (objfile *) alloc(sizeof(objfile));   // Alloue la structure
 object->size = filelength(fileno(of));
 object->raw = alloc(object->size);
 object->name = 0;

 if (fread(object->raw,1,object->size,of) == -1)
 {  // Charge le fichier
  fprintf(stderr,"Can't read object file %s\n",name);
  free(object->raw);
  free(object);
  fclose(of);
  return 0;
 }

 fclose(of);
 hdr = object->raw;

 if (rus(hdr->f_magic) == 0x150)
 {
  object->type = COFF;
  if (rus(hdr->f_nscns) != 3)
  {
   fprintf(stderr,"Bad number of sections in file: %s\n",name);
   delete_objfile(object);
   return 0;
  }
  if (hdr->f_opthdr)
  {
   fprintf(stderr,"Size of optional header is nonzero in file: %s\n",name);
   delete_objfile(object);
   return 0;
  }
 }
 else
 {
  object->type = AMDOS;
  if  (rul(*(long*)(object->raw+object->size-4)) != HunkEnd)
  {
   fprintf(stderr,"File %s is not valid\n",name);
   delete_objfile(object);
   return 0;
  }
 }
 object->name = alloc(strlen(name)+1);
 strcpy(object->name,name);
 return object;
}

void delete_objfile(objfile* object)
{
 free(object->raw);
 free(object);
}

int exist(char *name)
{
 FILE *fp;
 fp = fopen(name,"rb");
 if (!fp) return 0;
 fclose(fp);
 return 1;
}

void *alloc(size_t size)
{
 void *ptr = (void *) malloc(size);
 if (!ptr)
 {
  fputs("alloc: Not enough memory to link\n",stderr);
  exit(1);            // A AMELIORER PLUS TARD
 }
 return ptr;
}

void *re_alloc(void *ptr,size_t size)
{
 ptr = (void *) realloc(ptr,size);
 if (!ptr)
 {
  fputs("re_alloc: Not enough memory to link\n",stderr);
  exit(1);            // A AMELIORER PLUS TARD
 }
 return ptr;
}

void del_extension(char *name)
{
 char *seeker = name + strlen(name) - 1;
 while (seeker>=name)
 {
  if (*seeker=='.')
  {
   *seeker=0;   // Si on trouve le '.' on termine la chaine ici
   break;
  }
  seeker--;
 }
}

unsigned short rus(unsigned short uwptr)
{
 return (uwptr << 8) | (uwptr>>8);
}

ushort* rusp(ushort* usp)
{
 *usp = rus(*usp);
 return usp;
}

ulong* rulp(ulong* ulp)
{
 *ulp = rul(*ulp);
 return ulp;
}

unsigned long rul(unsigned long ul)
{
 char *ptr = (char *) &ul;
 register char tmp;
 tmp = ptr[3];
 ptr[3] = ptr[0];
 ptr[0] = tmp;
 tmp = ptr[2];
 ptr[2] = ptr[1];
 ptr[1] = tmp;
 return *(long*)ptr;
}

int main(int argc, char **argv)
{
 char **objnames = 0;       // nom des fichiers objets
 int objnb = 0;
 objfile *object;          // obj brut
 objimage *destobj;
 objimage *obj;             // obj traduit

 TIfile *compiled;          // fichier TIfile
 int i;
 int error=0;

 quiet = FALSE;

 // Look for -q switch
 for (i=0; i<argc; i++)
 {
   if (!strcmp(argv[i],"-q"))
     quiet = TRUE;
 }

 if(!quiet)
 {
  puts("Link 1.1       by Xavier VASSOR  xvassor@mail.dotcom.fr");
  puts("Minor bugfixes by Niklas Brunlid  e96nbr@efd.lth.se");
 }

 if (argc == 1)
 {
  fprintf(stderr,"%s: no arguments\n",argv[0]);
  error = 1;
 }
 else
  for (i=1;i<argc;i++)
  {
   if (argv[i][0] == '-')
   {
    switch (argv[i][1])
    {
     case 'q':
      break;

     case 'o':    // si c l'indic du nom du projet
      i++;
      project = alloc(strlen(argv[i])+1);
      strcpy(project,argv[i]);
      break;

     default:
      fprintf(stderr,"Error: unrecognized option %c\n",argv[i][1]);
      error = 1;
    }
   }
   else
   {
    if (!exist(argv[i]))
    {
     fprintf(stderr,"cannot open %s: No such file or directory\n",argv[i]);
     error=1;
    }
    else
    {
     objnb++;
     objnames = re_alloc(objnames,4*objnb);
     objnames[objnb-1] = alloc(strlen(argv[i])+1);
     strcpy(objnames[objnb-1],argv[i]);
    }
   }
  }

 if (error)
 {
  for (i=0;i<objnb;i++)
   free(objnames[i]);
  free (objnames);
  return 1;
 }

 if (!project)
 {
  project = alloc(strlen(objnames[0])+1);
  strcpy(project,objnames[0]);
 }
 del_extension(project);
 for (i=0; i<strlen(project); i++)
 {
   if (isupper(project[i]))
     project[i] -= ('A'-'a');
 }

 if (strlen(project) > 8)
 {
  fprintf(stderr,"Destination file name %s is too long\n",project);
  return 1;
 }

                // Construit un fichier objet 'racine' //
 destobj = create_objimage();

 for (i=0;i<objnb;i++)
 {
  object = read_object(objnames[i]);
  if (!object)
  {
   delete_objimage(destobj);
   for (i=0;i<objnb;i++)
    free(objnames[i]);
   free (objnames);
   return 1;
  }
  obj = translate_object(object);
  delete_objfile(object);
  if (!obj)
  {
   delete_objimage(destobj);
   for (i=0;i<objnb;i++)
    free(objnames[i]);
   free (objnames);
   return 1;
  }

  if (!append_object(destobj,obj))
  {
   delete_objimage(destobj);
   for (i=0;i<objnb;i++)
    free(objnames[i]);
   free (objnames);
   return 1;
  }
 }

 compiled = object_to_TI(destobj);    // tranforme en format DoorsOS/ Plusshell
 delete_objimage(destobj);                     // supprime le COFF traduit
 for (i=0;i<objnb;i++)
   free(objnames[i]);
 free (objnames);

 if (!compiled)
   return 1;

 if (!write_TIfile(compiled,project)) // ecrit le(les) fichier(s) final(finaux)
 {
  delete_TIfile(compiled);
  return 1;
 }

 delete_TIfile(compiled);
 return 0;
}
