#ifndef OBJIMAGE_H
#define OBJIMAGE_H

typedef struct{
  char *name;
  ulong offset;
  short section;
  long  symindex;  // dans la table originale
}spublic;

typedef struct{
  char *name;
  ulong size;   // taille ds cette partie
  long  symindex;  // dans la table originale
}bsssym;

typedef struct{
  char *name;
  long  symindex;    // dans la table originale
  int enabled;       // dit si il est encore 'actif'
  ushort pubsection; // corresponding public sym sections after it has been resolved
  ulong puboffs;
}unresolved;

typedef struct{
  ulong offset;
  ushort section;
}prv_rel;

typedef struct{
  ulong offset;
  bsssym *sym;       // Necessaire pour le COFF
}bss_rel;

typedef struct{
  ulong offset;
  unresolved *sym;
  ushort type; // 0 = 32bits, 1 = 16 bits
}unr_rel;

typedef struct{
  ushort nb_prv;
  ushort nb_unr;
  ushort nb_bss;
  prv_rel **prv; // reloc table
  unr_rel **unr; // ROM calls, RAM calls, libs
  bss_rel **bss; // BSS
}sct_rel;

typedef struct{
  ushort nb_pub;
  ushort nb_unr;
  ushort nb_bss;
  spublic **pub_sym;    // export,comment, main, exit, flags
  unresolved **unr_sym; // libs, ROM CALLS, RAM CALLS
  bsssym **bss_sym;
}sct_sym;

typedef struct{
  ulong size;
  sct_rel rel;
  char *raw;
}section;

typedef struct{
  section sections[3];	// Pointeur vers un tableau de sections
/*                         section[0] : .text
                           section[1] : .data
                           section[2] : .BSS            */
  sct_sym symbs;        // Symbol Table
}objimage;

typedef struct{
  int size;
  char *raw;
  int type;
  char *name;
}objfile;

#define COFF  0x00
#define AMDOS 0x01

// Defines from the A68K source

/* Hunk number definitions */
#define HunkNone 0	/* Not in a hunk */
#define HunkUnit 0x3E7
#define HunkName 0x3E8
#define HunkCode 0x3E9
#define HunkData 0x3EA
#define HunkBSS  0x3EB

#define HunkR32  0x3EC
#define HunkR16  0x3ED
#define HunkR8   0x3EE
#define HunkExt  0x3EF
#define HunkSym  0x3F0
#define HunkDbg  0x3F1
#define HunkEnd  0x3F2

typedef struct{
 int type;    // Code, data, ou bss
 int no;      // No de hunk
 int size;    // Taille relle
 int offset;  // offset par rapport aux hunks du meme type
}hunk;

#define CODE 0
#define DATA 1
#define BSS  2

objimage* create_objimage(void);
objimage* translate_object(objfile* object);
void delete_objimage(objimage*);
objimage* append_object(objimage*, objimage*); // fusionne 2 objets


#endif
