#include <stdlib.h>
#include <string.h>
#include "m68k.h"
#include "doorsos.h"
#include "objimage.h"
#include "link.h"

extern int nbsym,nbrel,i,nbok;       // nb de items trouvs
extern struct external_syment *symb;
extern struct external_reloc *rel;
extern struct external_filehdr *hdr;
extern char *strtable;

int is_sym_sec(struct external_syment *symb)
{
 if ((symb->e_sclass==3) && (rus(symb->e_scnum) > 0) && (symb->e_numaux))
  return 1;
 return 0;
}

int is_sym_pub(struct external_syment *symb)
{
 if ((symb->e_sclass==2) && (rus(symb->e_scnum) > 0))
   return 1;
 if ((symb->e_sclass==2) && (rus(symb->e_scnum)==0) && (rul(symb->e_value) != 0))
   if (is_bss_pub(symb))
     return 1;
 return 0;
}

int is_sym_unr(struct external_syment *symb)
{
 if ((symb->e_sclass==2) && (rus(symb->e_scnum)==0) && (rul(symb->e_value) == 0))
   return 1;
 if ((symb->e_sclass==2) && (rus(symb->e_scnum)==0) && (rul(symb->e_value) != 0))
   if (is_bss_unr(symb))
     return 1;
 return 0;
}

int is_sym_bss(struct external_syment *symb)
{
 if ((symb->e_sclass==2) && (rus(symb->e_scnum)==0) && (rul(symb->e_value) != 0))
 {
  if (is_bss_true(symb))
   return 1;
 }
 return 0;
}

int is_bss_true(struct external_syment *symb)
{
 char *str = get_sym_name(symb);
 int res = !strncmp(str,"_RAM_CALL_",10) ||
           !strncmp(str,"_ROM_CALL_",10) ||
           is_lib_name(str)         ||
           !strcmp(str,"_ti89")     ||
           !strcmp(str,"_ti92plus") ||
           !strcmp(str,"_library")  ||
           !strcmp(str,"_nostub")  ||
           !strcmp(str,"_prosit")  ||
           !strcmp(str,"comment");
 free (str);
 if (res)
  return 0;
 return 1;
}

int is_bss_unr(struct external_syment *symb)
{
 char *str = get_sym_name(symb);
 int res = !strncmp(str,"_RAM_CALL_",10) ||
           !strncmp(str,"_ROM_CALL_",10) ||
           is_lib_name(str);

 free (str);
 if (res)
  return 1;
 return 0;
}

int is_bss_pub(struct external_syment *symb)
{
 char *str = get_sym_name(symb);
 int res = !strcmp(str,"_ti89") ||
           !strcmp(str,"_ti92plus") ||
           !strcmp(str,"_library")  ||
           !strcmp(str,"_nostub")  ||
           !strcmp(str,"_prosit")  ||
           !strcmp(str,"comment");
 free (str);
 return res? 1 : 0;
}

char *get_sym_name(struct external_syment *symb)
{
 char *str;
 if (symb->e.e.e_zeroes)                                // si le nom est pas ds strtable
 {
  str = alloc(9);                // 8 chars + \0
  strncpy(str,symb->e.e_name,8); // copie le nom
  str[8] = 0;
 }
 else
 {
  char *ptr = strtable + rul(symb->e.e.e_offset);       // pointeur vers le nom dans la table
  str = alloc(strlen(ptr)+1); // alloue
  strcpy(str,ptr);            // copie le nom
 }
 return str;
}

int nb_pub_sym(objfile* object)
{
 nbok = 0;
 hdr  = object->raw;
 nbsym= rul(hdr->f_nsyms);
 symb = object->raw+rul(hdr->f_symptr);
 for (i=0;i<nbsym; i++,symb = (char *)(symb)+18)
  if (is_sym_pub(symb))
   nbok++;

 return nbok;
}

int nb_unr_sym(objfile* object)
{
 nbok = 0;
 hdr  = object->raw;
 nbsym= rul(hdr->f_nsyms);
 symb = object->raw+rul(hdr->f_symptr);
 for (i=0;i<nbsym;i++,symb=(char *)(symb)+18)
 {
  if (is_sym_unr(symb))
   nbok++;
 }
 return nbok;
}

int nb_bss_sym(objfile* object)
{
 nbok = 0;
 hdr  = object->raw;
 nbsym= rul(hdr->f_nsyms);
 symb = object->raw+rul(hdr->f_symptr);
 for (i=0;i<nbsym;i++,symb=(char *)(symb)+18)
 {
  if (is_sym_bss(symb))
   nbok++;
 }
 return nbok;
}

int nb_prv_rel(struct external_scnhdr* scnhdr, objfile* object)
{
 nbok = 0;
 hdr  = object->raw;
 nbsym= rul(hdr->f_nsyms);
 symb = object->raw+rul(hdr->f_symptr);

 nbrel = rus(scnhdr->s_nreloc);
 rel = object->raw+rul(scnhdr->s_relptr);

 for (i=0 ;i<nbrel; i++,rel=(char *)(rel)+10)
 {
  struct external_syment* sym = (char *)symb+18*rul(rel->r_symndx);
  if (is_sym_sec(sym) || is_sym_pub(sym))
   nbok++;
 }
 return nbok;
}

int nb_unr_rel(struct external_scnhdr* scnhdr, objfile* object)
{
 nbok = 0;
 hdr  = object->raw;
 nbsym= rul(hdr->f_nsyms);
 symb = object->raw+rul(hdr->f_symptr);

 nbrel = rus(scnhdr->s_nreloc);
 rel = object->raw+rul(scnhdr->s_relptr);

 for (i=0;i<nbrel;i++,rel=(char *)(rel)+10)
 {
  if (is_sym_unr((char *)symb+18*rul(rel->r_symndx)))
   nbok++;
 }
 return nbok;
}

int nb_bss_rel(struct external_scnhdr* scnhdr, objfile* object)
{
 nbok = 0;
 hdr  = object->raw;
 nbsym= rul(hdr->f_nsyms);
 symb = object->raw+rul(hdr->f_symptr);

 nbrel = rus(scnhdr->s_nreloc);
 rel = object->raw+rul(scnhdr->s_relptr);

 for (i=0;i<nbrel;i++,rel=(char *)(rel)+10)
 {
  if (is_sym_bss((char *)symb+18*rul(rel->r_symndx)))
   nbok++;
 }
 return nbok;
}

int is_lib_name(char *str)     // ! : gre pas les lib@0000 (?)
{
 int k;
 k = strlen(str);
 if ( k<7 || k>14 ) return 0;  // pas un nom de lib si taille<7 ou taille>14
 if (strncmp("__",str+k-6,2)) return 0; // recherche du '__'
 return 1;       			   // sinon c une lib
}

